@@ -18,6 +18,8 @@
 //! This module contains two main structs: [Buffer] and [MutableBuffer]. A buffer represents
 //! a contiguous memory region that can be shared via `offsets`.
 
+mod offset;
+pub use offset::*;
 mod immutable;
 pub use immutable::*;
 mod mutable;
