@@ -21,6 +21,7 @@
 
 use crate::raw::boolean_array::BooleanArrayDecoder;
 use crate::raw::list_array::ListArrayDecoder;
+use crate::raw::map_array::MapArrayDecoder;
 use crate::raw::primitive_array::PrimitiveArrayDecoder;
 use crate::raw::string_array::StringArrayDecoder;
 use crate::raw::struct_array::StructArrayDecoder;
@@ -33,6 +34,7 @@ use std::io::BufRead;
 
 mod boolean_array;
 mod list_array;
+mod map_array;
 mod primitive_array;
 mod string_array;
 mod struct_array;
@@ -41,6 +43,7 @@ mod tape;
 /// A builder for [`RawReader`] and [`RawDecoder`]
 pub struct RawReaderBuilder {
     batch_size: usize,
+    coerce_primitive: bool,
 
     schema: SchemaRef,
 }
@@ -56,6 +59,7 @@ impl RawReaderBuilder {
     pub fn new(schema: SchemaRef) -> Self {
         Self {
             batch_size: 1024,
+            coerce_primitive: false,
             schema,
         }
     }
@@ -65,6 +69,14 @@ impl RawReaderBuilder {
         Self { batch_size, ..self }
     }
 
+    /// Sets if the decoder should coerce primitive values (bool and number) into string when the Schema's column is Utf8 or LargeUtf8.
+    pub fn coerce_primitive(self, coerce_primitive: bool) -> Self {
+        Self {
+            coerce_primitive,
+            ..self
+        }
+    }
+
     /// Create a [`RawReader`] with the provided [`BufRead`]
     pub fn build<R: BufRead>(self, reader: R) -> Result<RawReader<R>, ArrowError> {
         Ok(RawReader {
@@ -75,7 +87,11 @@ impl RawReaderBuilder {
 
     /// Create a [`RawDecoder`]
     pub fn build_decoder(self) -> Result<RawDecoder, ArrowError> {
-        let decoder = make_decoder(DataType::Struct(self.schema.fields.clone()), false)?;
+        let decoder = make_decoder(
+            DataType::Struct(self.schema.fields.clone()),
+            self.coerce_primitive,
+            false,
+        )?;
         let num_fields = self.schema.all_fields().len();
 
         Ok(RawDecoder {
@@ -268,6 +284,7 @@ macro_rules! primitive_decoder {
 
 fn make_decoder(
     data_type: DataType,
+    coerce_primitive: bool,
     is_nullable: bool,
 ) -> Result<Box<dyn ArrayDecoder>, ArrowError> {
     downcast_integer! {
@@ -275,14 +292,15 @@ fn make_decoder(
         DataType::Float32 => primitive_decoder!(Float32Type, data_type),
         DataType::Float64 => primitive_decoder!(Float64Type, data_type),
         DataType::Boolean => Ok(Box::<BooleanArrayDecoder>::default()),
-        DataType::Utf8 => Ok(Box::<StringArrayDecoder::<i32>>::default()),
-        DataType::LargeUtf8 => Ok(Box::<StringArrayDecoder::<i64>>::default()),
-        DataType::List(_) => Ok(Box::new(ListArrayDecoder::<i32>::new(data_type, is_nullable)?)),
-        DataType::LargeList(_) => Ok(Box::new(ListArrayDecoder::<i64>::new(data_type, is_nullable)?)),
-        DataType::Struct(_) => Ok(Box::new(StructArrayDecoder::new(data_type, is_nullable)?)),
+        DataType::Utf8 => Ok(Box::new(StringArrayDecoder::<i32>::new(coerce_primitive))),
+        DataType::LargeUtf8 => Ok(Box::new(StringArrayDecoder::<i64>::new(coerce_primitive))),
+        DataType::List(_) => Ok(Box::new(ListArrayDecoder::<i32>::new(data_type, coerce_primitive, is_nullable)?)),
+        DataType::LargeList(_) => Ok(Box::new(ListArrayDecoder::<i64>::new(data_type, coerce_primitive, is_nullable)?)),
+        DataType::Struct(_) => Ok(Box::new(StructArrayDecoder::new(data_type, coerce_primitive, is_nullable)?)),
         DataType::Binary | DataType::LargeBinary | DataType::FixedSizeBinary(_) => {
             Err(ArrowError::JsonError(format!("{data_type} is not supported by JSON")))
         }
+        DataType::Map(_, _) => Ok(Box::new(MapArrayDecoder::new(data_type, coerce_primitive, is_nullable)?)),
         d => Err(ArrowError::NotYetImplemented(format!("Support for {d} in JSON reader")))
     }
 }
@@ -292,28 +310,36 @@ fn tape_error(d: TapeElement, expected: &str) -> ArrowError {
 }
 
 #[cfg(test)]
+#[allow(deprecated)]
 mod tests {
     use super::*;
     use crate::reader::infer_json_schema;
     use crate::ReaderBuilder;
     use arrow_array::cast::{
-        as_boolean_array, as_largestring_array, as_list_array, as_primitive_array,
-        as_string_array, as_struct_array,
+        as_boolean_array, as_largestring_array, as_list_array, as_map_array,
+        as_primitive_array, as_string_array, as_struct_array,
     };
     use arrow_array::types::Int32Type;
     use arrow_array::Array;
+    use arrow_cast::display::{ArrayFormatter, FormatOptions};
     use arrow_schema::{DataType, Field, Schema};
     use std::fs::File;
     use std::io::{BufReader, Cursor, Seek};
     use std::sync::Arc;
 
-    fn do_read(buf: &str, batch_size: usize, schema: SchemaRef) -> Vec<RecordBatch> {
+    fn do_read(
+        buf: &str,
+        batch_size: usize,
+        coerce_primitive: bool,
+        schema: SchemaRef,
+    ) -> Vec<RecordBatch> {
         let mut unbuffered = vec![];
 
         // Test with different batch sizes to test for boundary conditions
         for batch_size in [1, 3, 100, batch_size] {
             unbuffered = RawReaderBuilder::new(schema.clone())
                 .with_batch_size(batch_size)
+                .coerce_primitive(coerce_primitive)
                 .build(Cursor::new(buf.as_bytes()))
                 .unwrap()
                 .collect::<Result<Vec<_>, _>>()
@@ -327,6 +353,7 @@ mod tests {
             for b in [1, 3, 5] {
                 let buffered = RawReaderBuilder::new(schema.clone())
                     .with_batch_size(batch_size)
+                    .coerce_primitive(coerce_primitive)
                     .build(BufReader::with_capacity(b, Cursor::new(buf.as_bytes())))
                     .unwrap()
                     .collect::<Result<Vec<_>, _>>()
@@ -356,7 +383,7 @@ mod tests {
             Field::new("c", DataType::Boolean, true),
         ]));
 
-        let batches = do_read(buf, 1024, schema);
+        let batches = do_read(buf, 1024, false, schema);
         assert_eq!(batches.len(), 1);
 
         let col1 = as_primitive_array::<Int64Type>(batches[0].column(0));
@@ -393,7 +420,7 @@ mod tests {
             Field::new("b", DataType::LargeUtf8, true),
         ]));
 
-        let batches = do_read(buf, 1024, schema);
+        let batches = do_read(buf, 1024, false, schema);
         assert_eq!(batches.len(), 1);
 
         let col1 = as_string_array(batches[0].column(0));
@@ -450,7 +477,7 @@ mod tests {
             ),
         ]));
 
-        let batches = do_read(buf, 1024, schema);
+        let batches = do_read(buf, 1024, false, schema);
         assert_eq!(batches.len(), 1);
 
         let list = as_list_array(batches[0].column(0).as_ref());
@@ -513,7 +540,7 @@ mod tests {
             ),
         ]));
 
-        let batches = do_read(buf, 1024, schema);
+        let batches = do_read(buf, 1024, false, schema);
         assert_eq!(batches.len(), 1);
 
         let nested = as_struct_array(batches[0].column(0).as_ref());
@@ -541,6 +568,47 @@ mod tests {
         assert!(c.is_null(1));
     }
 
+    #[test]
+    fn test_map() {
+        let buf = r#"
+           {"map": {"a": ["foo", null]}}
+           {"map": {"a": [null], "b": []}}
+           {"map": {"c": null, "a": ["baz"]}}
+        "#;
+        let list = DataType::List(Box::new(Field::new("element", DataType::Utf8, true)));
+        let entries = DataType::Struct(vec![
+            Field::new("key", DataType::Utf8, false),
+            Field::new("value", list, true),
+        ]);
+
+        let map = DataType::Map(Box::new(Field::new("entries", entries, true)), false);
+        let schema = Arc::new(Schema::new(vec![Field::new("map", map, true)]));
+
+        let batches = do_read(buf, 1024, false, schema);
+        assert_eq!(batches.len(), 1);
+
+        let map = as_map_array(batches[0].column(0).as_ref());
+        let map_keys = as_string_array(map.keys().as_ref());
+        let map_values = as_list_array(map.values().as_ref());
+        assert_eq!(map.value_offsets(), &[0, 1, 3, 5]);
+
+        let k: Vec<_> = map_keys.iter().map(|x| x.unwrap()).collect();
+        assert_eq!(&k, &["a", "a", "b", "c", "a"]);
+
+        let list_values = as_string_array(map_values.values().as_ref());
+        let lv: Vec<_> = list_values.iter().collect();
+        assert_eq!(&lv, &[Some("foo"), None, None, Some("baz")]);
+        assert_eq!(map_values.value_offsets(), &[0, 2, 3, 3, 3, 4]);
+        assert_eq!(map_values.null_count(), 1);
+        assert!(map_values.is_null(3));
+
+        let options = FormatOptions::default().with_null("null");
+        let formatter = ArrayFormatter::try_new(map, &options).unwrap();
+        assert_eq!(formatter.value(0).to_string(), "{a: [foo, null]}");
+        assert_eq!(formatter.value(1).to_string(), "{a: [null], b: []}");
+        assert_eq!(formatter.value(2).to_string(), "{c: null, a: [baz]}");
+    }
+
     #[test]
     fn integration_test() {
         let files = [
@@ -567,4 +635,84 @@ mod tests {
             assert_eq!(a_result, b_result);
         }
     }
+
+    #[test]
+    fn test_not_coercing_primitive_into_string_without_flag() {
+        let schema = Arc::new(Schema::new(vec![Field::new("a", DataType::Utf8, true)]));
+
+        let buf = r#"{"a": 1}"#;
+        let result = RawReaderBuilder::new(schema.clone())
+            .with_batch_size(1024)
+            .build(Cursor::new(buf.as_bytes()))
+            .unwrap()
+            .read();
+
+        assert!(result.is_err());
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "Json error: expected string got number".to_string()
+        );
+
+        let buf = r#"{"a": true}"#;
+        let result = RawReaderBuilder::new(schema)
+            .with_batch_size(1024)
+            .build(Cursor::new(buf.as_bytes()))
+            .unwrap()
+            .read();
+
+        assert!(result.is_err());
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "Json error: expected string got true".to_string()
+        );
+    }
+
+    #[test]
+    fn test_coercing_primitive_into_string() {
+        let buf = r#"
+        {"a": 1, "b": 2, "c": true}
+        {"a": 2E0, "b": 4, "c": false}
+
+        {"b": 6, "a": 2.0}
+        {"b": "5", "a": 2}
+        {"b": 4e0}
+        {"b": 7, "a": null}
+        "#;
+
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("a", DataType::Utf8, true),
+            Field::new("b", DataType::Utf8, true),
+            Field::new("c", DataType::Utf8, true),
+        ]));
+
+        let batches = do_read(buf, 1024, true, schema);
+        assert_eq!(batches.len(), 1);
+
+        let col1 = as_string_array(batches[0].column(0));
+        assert_eq!(col1.null_count(), 2);
+        assert_eq!(col1.value(0), "1");
+        assert_eq!(col1.value(1), "2E0");
+        assert_eq!(col1.value(2), "2.0");
+        assert_eq!(col1.value(3), "2");
+        assert!(col1.is_null(4));
+        assert!(col1.is_null(5));
+
+        let col2 = as_string_array(batches[0].column(1));
+        assert_eq!(col2.null_count(), 0);
+        assert_eq!(col2.value(0), "2");
+        assert_eq!(col2.value(1), "4");
+        assert_eq!(col2.value(2), "6");
+        assert_eq!(col2.value(3), "5");
+        assert_eq!(col2.value(4), "4e0");
+        assert_eq!(col2.value(5), "7");
+
+        let col3 = as_string_array(batches[0].column(2));
+        assert_eq!(col3.null_count(), 4);
+        assert_eq!(col3.value(0), "true");
+        assert_eq!(col3.value(1), "false");
+        assert!(col3.is_null(2));
+        assert!(col3.is_null(3));
+        assert!(col3.is_null(4));
+        assert!(col3.is_null(5));
+    }
 }
