@@ -142,13 +142,9 @@ impl Field {
     }
 
     /// Sets the `Field`'s optional custom metadata.
-    /// The metadata is set as `None` for empty map.
     #[inline]
     pub fn set_metadata(&mut self, metadata: HashMap<String, String>) {
-        self.metadata = HashMap::default();
-        if !metadata.is_empty() {
-            self.metadata = metadata;
-        }
+        self.metadata = metadata;
     }
 
     /// Sets the metadata of this `Field` to be `metadata` and returns self
