@@ -266,10 +266,9 @@ fn like<'a, S: ArrayAccessor<Item = &'a str>>(
     right: S,
 ) -> Result<BooleanArray, ArrowError> {
     regex_like(left, right, false, |re_pattern| {
-        Regex::new(&format!("^{}$", re_pattern)).map_err(|e| {
+        Regex::new(&format!("^{re_pattern}$")).map_err(|e| {
             ArrowError::ComputeError(format!(
-                "Unable to build regex from LIKE pattern: {}",
-                e
+                "Unable to build regex from LIKE pattern: {e}"
             ))
         })
     })
@@ -313,10 +312,9 @@ fn like_scalar_op<'a, F: Fn(bool) -> bool, L: ArrayAccessor<Item = &'a str>>(
         }))
     } else {
         let re_pattern = replace_like_wildcards(right)?;
-        let re = Regex::new(&format!("^{}$", re_pattern)).map_err(|e| {
+        let re = Regex::new(&format!("^{re_pattern}$")).map_err(|e| {
             ArrowError::ComputeError(format!(
-                "Unable to build regex from LIKE pattern: {}",
-                e
+                "Unable to build regex from LIKE pattern: {e}"
             ))
         })?;
 
@@ -397,10 +395,9 @@ fn nlike<'a, S: ArrayAccessor<Item = &'a str>>(
     right: S,
 ) -> Result<BooleanArray, ArrowError> {
     regex_like(left, right, true, |re_pattern| {
-        Regex::new(&format!("^{}$", re_pattern)).map_err(|e| {
+        Regex::new(&format!("^{re_pattern}$")).map_err(|e| {
             ArrowError::ComputeError(format!(
-                "Unable to build regex from LIKE pattern: {}",
-                e
+                "Unable to build regex from LIKE pattern: {e}"
             ))
         })
     })
@@ -445,10 +442,9 @@ fn ilike<'a, S: ArrayAccessor<Item = &'a str>>(
     right: S,
 ) -> Result<BooleanArray, ArrowError> {
     regex_like(left, right, false, |re_pattern| {
-        Regex::new(&format!("(?i)^{}$", re_pattern)).map_err(|e| {
+        Regex::new(&format!("(?i)^{re_pattern}$")).map_err(|e| {
             ArrowError::ComputeError(format!(
-                "Unable to build regex from ILIKE pattern: {}",
-                e
+                "Unable to build regex from ILIKE pattern: {e}"
             ))
         })
     })
@@ -491,11 +487,8 @@ fn ilike_scalar_op<O: OffsetSizeTrait, F: Fn(bool) -> bool>(
     }
 
     let re_pattern = replace_like_wildcards(right)?;
-    let re = Regex::new(&format!("(?i)^{}$", re_pattern)).map_err(|e| {
-        ArrowError::ComputeError(format!(
-            "Unable to build regex from ILIKE pattern: {}",
-            e
-        ))
+    let re = Regex::new(&format!("(?i)^{re_pattern}$")).map_err(|e| {
+        ArrowError::ComputeError(format!("Unable to build regex from ILIKE pattern: {e}"))
     })?;
 
     Ok(BooleanArray::from_unary(left, |item| op(re.is_match(item))))
@@ -537,10 +530,9 @@ fn nilike<'a, S: ArrayAccessor<Item = &'a str>>(
     right: S,
 ) -> Result<BooleanArray, ArrowError> {
     regex_like(left, right, true, |re_pattern| {
-        Regex::new(&format!("(?i)^{}$", re_pattern)).map_err(|e| {
+        Regex::new(&format!("(?i)^{re_pattern}$")).map_err(|e| {
             ArrowError::ComputeError(format!(
-                "Unable to build regex from ILIKE pattern: {}",
-                e
+                "Unable to build regex from ILIKE pattern: {e}"
             ))
         })
     })
