@@ -36,7 +36,7 @@ fn do_bench(c: &mut Criterion, name: &str, cols: Vec<ArrayRef>) {
         .map(|x| SortField::new(x.data_type().clone()))
         .collect();
 
-    c.bench_function(&format!("convert_columns {}", name), |b| {
+    c.bench_function(&format!("convert_columns {name}"), |b| {
         b.iter(|| {
             let mut converter = RowConverter::new(fields.clone()).unwrap();
             black_box(converter.convert_columns(&cols).unwrap())
@@ -46,11 +46,11 @@ fn do_bench(c: &mut Criterion, name: &str, cols: Vec<ArrayRef>) {
     let mut converter = RowConverter::new(fields).unwrap();
     let rows = converter.convert_columns(&cols).unwrap();
     // using a pre-prepared row converter should be faster than the first time
-    c.bench_function(&format!("convert_columns_prepared {}", name), |b| {
+    c.bench_function(&format!("convert_columns_prepared {name}"), |b| {
         b.iter(|| black_box(converter.convert_columns(&cols).unwrap()));
     });
 
-    c.bench_function(&format!("convert_rows {}", name), |b| {
+    c.bench_function(&format!("convert_rows {name}"), |b| {
         b.iter(|| black_box(converter.convert_rows(&rows).unwrap()));
     });
 }
