@@ -386,11 +386,11 @@ fn create_list_array<U: AsRef<[i32]>, T: AsRef<[Option<U>]>>(data: T) -> ArrayDa
 
 #[test]
 fn test_list_equal() {
-    let a = create_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
-    let b = create_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
+    let a = create_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
+    let b = create_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
     test_equal(&a, &b, true);
 
-    let b = create_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 7])]);
+    let b = create_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 7])]);
     test_equal(&a, &b, false);
 }
 
@@ -448,11 +448,11 @@ fn test_empty_offsets_list_equal() {
 // Test the case where null_count > 0
 #[test]
 fn test_list_null() {
-    let a = create_list_array(&[Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
-    let b = create_list_array(&[Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
+    let a = create_list_array([Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
+    let b = create_list_array([Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
     test_equal(&a, &b, true);
 
-    let b = create_list_array(&[
+    let b = create_list_array([
         Some(&[1, 2]),
         None,
         Some(&[5, 6]),
@@ -462,7 +462,7 @@ fn test_list_null() {
     ]);
     test_equal(&a, &b, false);
 
-    let b = create_list_array(&[Some(&[1, 2]), None, None, Some(&[3, 5]), None, None]);
+    let b = create_list_array([Some(&[1, 2]), None, None, Some(&[3, 5]), None, None]);
     test_equal(&a, &b, false);
 
     // a list where the nullness of values is determined by the list's bitmap
@@ -506,8 +506,8 @@ fn test_list_null() {
 // Test the case where offset != 0
 #[test]
 fn test_list_offsets() {
-    let a = create_list_array(&[Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
-    let b = create_list_array(&[Some(&[1, 2]), None, None, Some(&[3, 5]), None, None]);
+    let a = create_list_array([Some(&[1, 2]), None, None, Some(&[3, 4]), None, None]);
+    let b = create_list_array([Some(&[1, 2]), None, None, Some(&[3, 5]), None, None]);
 
     let a_slice = a.slice(0, 3);
     let b_slice = b.slice(0, 3);
@@ -539,32 +539,32 @@ fn create_fixed_size_binary_array<U: AsRef<[u8]>, T: AsRef<[Option<U>]>>(
 
 #[test]
 fn test_fixed_size_binary_equal() {
-    let a = create_fixed_size_binary_array(&[Some(b"hello"), Some(b"world")]);
-    let b = create_fixed_size_binary_array(&[Some(b"hello"), Some(b"world")]);
+    let a = create_fixed_size_binary_array([Some(b"hello"), Some(b"world")]);
+    let b = create_fixed_size_binary_array([Some(b"hello"), Some(b"world")]);
     test_equal(&a, &b, true);
 
-    let b = create_fixed_size_binary_array(&[Some(b"hello"), Some(b"arrow")]);
+    let b = create_fixed_size_binary_array([Some(b"hello"), Some(b"arrow")]);
     test_equal(&a, &b, false);
 }
 
 // Test the case where null_count > 0
 #[test]
 fn test_fixed_size_binary_null() {
-    let a = create_fixed_size_binary_array(&[Some(b"hello"), None, Some(b"world")]);
-    let b = create_fixed_size_binary_array(&[Some(b"hello"), None, Some(b"world")]);
+    let a = create_fixed_size_binary_array([Some(b"hello"), None, Some(b"world")]);
+    let b = create_fixed_size_binary_array([Some(b"hello"), None, Some(b"world")]);
     test_equal(&a, &b, true);
 
-    let b = create_fixed_size_binary_array(&[Some(b"hello"), Some(b"world"), None]);
+    let b = create_fixed_size_binary_array([Some(b"hello"), Some(b"world"), None]);
     test_equal(&a, &b, false);
 
-    let b = create_fixed_size_binary_array(&[Some(b"hello"), None, Some(b"arrow")]);
+    let b = create_fixed_size_binary_array([Some(b"hello"), None, Some(b"arrow")]);
     test_equal(&a, &b, false);
 }
 
 #[test]
 fn test_fixed_size_binary_offsets() {
     // Test the case where offset != 0
-    let a = create_fixed_size_binary_array(&[
+    let a = create_fixed_size_binary_array([
         Some(b"hello"),
         None,
         None,
@@ -572,7 +572,7 @@ fn test_fixed_size_binary_offsets() {
         None,
         None,
     ]);
-    let b = create_fixed_size_binary_array(&[
+    let b = create_fixed_size_binary_array([
         Some(b"hello"),
         None,
         None,
@@ -706,18 +706,18 @@ fn create_fixed_size_list_array<U: AsRef<[i32]>, T: AsRef<[Option<U>]>>(
 
 #[test]
 fn test_fixed_size_list_equal() {
-    let a = create_fixed_size_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
-    let b = create_fixed_size_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
+    let a = create_fixed_size_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
+    let b = create_fixed_size_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 6])]);
     test_equal(&a, &b, true);
 
-    let b = create_fixed_size_list_array(&[Some(&[1, 2, 3]), Some(&[4, 5, 7])]);
+    let b = create_fixed_size_list_array([Some(&[1, 2, 3]), Some(&[4, 5, 7])]);
     test_equal(&a, &b, false);
 }
 
 // Test the case where null_count > 0
 #[test]
 fn test_fixed_list_null() {
-    let a = create_fixed_size_list_array(&[
+    let a = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         None,
@@ -725,7 +725,7 @@ fn test_fixed_list_null() {
         None,
         None,
     ]);
-    let b = create_fixed_size_list_array(&[
+    let b = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         None,
@@ -735,7 +735,7 @@ fn test_fixed_list_null() {
     ]);
     test_equal(&a, &b, true);
 
-    let b = create_fixed_size_list_array(&[
+    let b = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         Some(&[7, 8, 9]),
@@ -745,7 +745,7 @@ fn test_fixed_list_null() {
     ]);
     test_equal(&a, &b, false);
 
-    let b = create_fixed_size_list_array(&[
+    let b = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         None,
@@ -755,7 +755,7 @@ fn test_fixed_list_null() {
     ]);
     test_equal(&a, &b, false);
 
-    let b = create_fixed_size_list_array(&[None, Some(&[4, 5, 6]), None, None]);
+    let b = create_fixed_size_list_array([None, Some(&[4, 5, 6]), None, None]);
 
     test_equal(&a.slice(2, 4), &b, true);
     test_equal(&a.slice(3, 3), &b.slice(1, 3), true);
@@ -764,7 +764,7 @@ fn test_fixed_list_null() {
 #[test]
 fn test_fixed_list_offsets() {
     // Test the case where offset != 0
-    let a = create_fixed_size_list_array(&[
+    let a = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         None,
@@ -772,7 +772,7 @@ fn test_fixed_list_offsets() {
         None,
         None,
     ]);
-    let b = create_fixed_size_list_array(&[
+    let b = create_fixed_size_list_array([
         Some(&[1, 2, 3]),
         None,
         None,
